﻿using System;
using System.Data;
using System.Data.SqlClient;
using System.Windows.Forms;
using System.Drawing;
using Excel = Microsoft.Office.Interop.Excel;

namespace LocalDb {
  public partial class Form1 : Form {
    private SqlDataAdapter DataAdapter;

    public Form1() { InitializeComponent(); }

    private void Form1_Load(
      object sender, EventArgs e) {

      this.Text = "名簿";
      dataGridView1.Dock = DockStyle.Fill;
      dataGridView1.DataSource = bindingSource1;

      const string connectString =
        @"Data Source=(LocalDB)\v11.0;" +
        "AttachDbFilename=" +
        @"|DataDirectory|\Person.mdf;";
      const string selectString = 
        "select * from TableMain";

      try {
        DataAdapter = new SqlDataAdapter(
          selectString, connectString);
        var commandBuilder =
          new SqlCommandBuilder(DataAdapter);
        UpdateGrid();
      } catch (Exception ex) {
        MessageBox.Show(ex.Message, "エラー");
      }

      printDialog1.Document = printDocument1;
      printPreviewDialog1.Document = printDocument1;
    }

    private void UpdateGrid() {
      var dataTable = new DataTable();
      DataAdapter.Fill(dataTable);
      bindingSource1.DataSource = dataTable;
      buttonSubmit.Enabled = false;
    }

    private void dataGridView1_CellValueChanged(
      object sender, DataGridViewCellEventArgs e) {

      buttonSubmit.Enabled = true;
    }

    private void dataGridView1_RowsRemoved(
      object sender,
      DataGridViewRowsRemovedEventArgs e) {

      buttonSubmit.Enabled = true;
    }

    private void buttonSubmit_Click(
      object sender, EventArgs e) {

      DataAdapter.Update(
        (DataTable)bindingSource1.DataSource);
      UpdateGrid();
    }

    private void buttonPrint_Click(
     object sender, EventArgs e) {

      if (DialogResult.OK ==
        printDialog1.ShowDialog()) {
        printDocument1.Print();
      }
    }

    private void buttonPrintPreview_Click(
      object sender, EventArgs e) {

      printPreviewDialog1.ShowDialog();
    }

    private void printDocument1_PrintPage(
      object sender,
      System.Drawing.Printing.PrintPageEventArgs e) {

      e.Graphics.PageUnit = GraphicsUnit.Millimeter;
      const int margin = 10;
      const int marginInCell = 2;
      int[] cellWidth = new int[] { 20, 50, 50 };
      int columnCount = cellWidth.Length;
      int cellWidthTotal = 0;
      foreach (var w in cellWidth)
        cellWidthTotal += w;
      int rowCount = dataGridView1.Rows.Count;
      const int cellHeight = 10;
      int cellHeightTotal = cellHeight * rowCount;
      var pen = new Pen(Color.Black, 0.2f);
      int x = margin, y;

      for (int i = 0; i <= rowCount; i++) {
        y = margin + i * cellHeight;
        e.Graphics.DrawLine(pen,
          margin, y, margin + cellWidthTotal, y);
      }

      for (int i = 0; i <= columnCount; i++) {
        e.Graphics.DrawLine(pen,
          x, margin, x, margin + cellHeightTotal);
        if (i != columnCount) x += cellWidth[i];
      }

      var brush = Brushes.Black;
      var font = new Font("ＭＳ 明朝", 16);
      x = margin + marginInCell;
      for (int i = 0; i < columnCount; i++) {
        y = margin + marginInCell;
        e.Graphics.DrawString(
          dataGridView1.Columns[i].HeaderText,
          font, brush, x, y);
        x += cellWidth[i];
      }

      y = margin + marginInCell;
      for (int i = 0; i < rowCount - 1; i++) {
        x = margin + marginInCell;
        y += cellHeight;
        for (int j = 0; j < columnCount; j++) {
          string s;
          var v = dataGridView1[j, i].Value;
          if (j != 2) s = v.ToString();
          else s = ((DateTime)v).ToShortDateString();
          e.Graphics.DrawString(s, font, brush, x, y);
          x += cellWidth[j];
        }
      }
      e.HasMorePages = false;
    }

    private void buttonExportToExcel_Click(
         object sender, EventArgs e) {

      Excel.Application xlApp =
        new Excel.Application();
      Excel.Workbook wb = xlApp.Workbooks.Add();
      Excel.Worksheet ws = wb.Sheets[1];
      Excel.Range range;
      string s;

      var dgv = dataGridView1;
      for (int i = 0; i < dgv.Rows.Count - 1; i++)
        for (int j = 0; j < dgv.Columns.Count; j++) {
          range = ws.Cells[i + 1, j + 1];
          if (j != 2)
            s = dgv[j, i].Value.ToString();
          else
            s = ((DateTime)dgv[j, i].Value).
              ToShortDateString();
          range.Value2 = s;
        }
      ws.UsedRange.EntireColumn.AutoFit();
      xlApp.WindowState = Excel.XlWindowState.xlNormal;
      xlApp.Visible = true;
    }

    private void buttonExportToTsv_Click(
          object sender, EventArgs e) {

      var sb = new System.Text.StringBuilder();
      string s;
      var dgv = dataGridView1;
      for (int i = 0; i < dgv.Rows.Count - 1; i++) {
        for (int j = 0; j < dgv.Columns.Count; j++) {
          var v = dgv[j, i].Value;
          if (j != 2) s = v.ToString() + "\t";
          else s = ((DateTime)v).ToShortDateString();
          sb.Append(s);
        }
        sb.AppendLine();
      }

      var todayString = DateTime.Now.Year.ToString() +
        DateTime.Now.Month.ToString() +
        DateTime.Now.Day.ToString();
      var fileName = System.IO.Path.Combine(
        System.Environment.GetFolderPath(
        Environment.SpecialFolder.MyDocuments),
        todayString + "ToExcel.tsv");
      System.IO.File.WriteAllText(fileName,
        sb.ToString(), System.Text.Encoding.
        GetEncoding("Shift_JIS"));
      System.Diagnostics.Process.Start(fileName);
    }

  }
}
