﻿Imports System.Data.SqlClient
Imports Microsoft.Office.Interop


Public Class Form1
  Private DataAdapter As SqlDataAdapter

  Private Sub Form1_Load(sender As Object, e As EventArgs) Handles MyBase.Load
    Me.Text = "名簿"
    DataGridView1.Dock = DockStyle.Fill
    DataGridView1.DataSource = BindingSource1
    Const connectString =
      "Data Source=(LocalDB)\v11.0;" &
      "AttachDbFilename=|DataDirectory|\Person.mdf;"
    Const selectString = "select * from TableMain"

    Try
      DataAdapter = New SqlDataAdapter(
        selectString, connectString)
      Dim commandBuilder =
        New SqlCommandBuilder(DataAdapter)
      UpdateGrid()

    Catch ex As Exception
      MessageBox.Show(ex.Message, "エラー")
    End Try

    PrintDialog1.Document = PrintDocument1
    PrintPreviewDialog1.Document = PrintDocument1
  End Sub

  Private Sub UpdateGrid()
    Dim dataTable = New DataTable()
    DataAdapter.Fill(dataTable)
    BindingSource1.DataSource = dataTable
    ButtonSubmit.Enabled = False
  End Sub

  Private Sub DataGridView1_CellValueChanged(sender As Object,
    e As DataGridViewCellEventArgs) Handles DataGridView1.CellValueChanged

    ButtonSubmit.Enabled = True
  End Sub

  Private Sub DataGridView1_RowsRemoved(sender As Object,
    e As DataGridViewRowsRemovedEventArgs) Handles DataGridView1.RowsRemoved

    ButtonSubmit.Enabled = True
  End Sub

  Private Sub ButtonSubmit_Click(sender As Object,
    e As EventArgs) Handles ButtonSubmit.Click

    DataAdapter.Update(BindingSource1.DataSource)
    UpdateGrid()
  End Sub

  Private Sub ButtonPrint_Click(sender As Object,
    e As EventArgs) Handles ButtonPrint.Click

    If DialogResult.OK = PrintDialog1.ShowDialog() Then
      PrintDocument1.Print()
    End If
  End Sub

  Private Sub ButtonPrintPreview_Click(sender As Object,
    e As EventArgs) Handles ButtonPrintPreview.Click

    PrintPreviewDialog1.ShowDialog()
  End Sub

  Private Sub PrintDocument1_PrintPage(sender As Object,
    e As Printing.PrintPageEventArgs) Handles PrintDocument1.PrintPage

    e.Graphics.PageUnit = GraphicsUnit.Millimeter
    Const margin = 10
    Const marginInCell = 2
    Dim cellWidth() = New Integer() {20, 50, 50}
    Dim columnCount = cellWidth.Length
    Dim cellWidthTotal = 0
    For Each w In cellWidth
      cellWidthTotal += w
    Next
    Dim rowCount = DataGridView1.Rows.Count
    Const cellHeight = 10
    Dim cellHeightTotal = cellHeight * rowCount
    Dim pen = New Pen(Color.Black, 0.2F)
    Dim x = margin
    Dim y As Integer

    For i = 0 To rowCount
      y = margin + i * cellHeight
      e.Graphics.DrawLine(pen, margin, y, margin + cellWidthTotal, y)
    Next

    For i = 0 To columnCount
      e.Graphics.DrawLine(pen, x, margin, x, margin + cellHeightTotal)
      If i <> columnCount Then x += cellWidth(i)
    Next

    Dim brush = Brushes.Black
    Dim font = New Font("ＭＳ 明朝", 16)
    x = margin + marginInCell
    For i = 0 To columnCount - 1
      y = margin + marginInCell
      e.Graphics.DrawString(
        DataGridView1.Columns(i).HeaderText, font, brush, x, y)
      x += cellWidth(i)
    Next

    y = margin + marginInCell
    For i = 0 To rowCount - 2
      x = margin + marginInCell
      y += cellHeight
      For j = 0 To columnCount - 1
        Dim s As String
        Dim v = DataGridView1(j, i).Value
        If j <> 2 Then
          s = v.ToString()
        Else
          s = CType(v, Date).ToShortDateString()
        End If
        e.Graphics.DrawString(s, font, brush, x, y)
        x += cellWidth(j)
      Next
    Next
    e.HasMorePages = False
  End Sub

  Private Sub ButtonExportToExcel_Click(sender As Object,
    e As EventArgs) Handles ButtonExportToExcel.Click

    Dim xlApp = New Excel.Application()
    Dim wb = xlApp.Workbooks.Add()
    Dim ws = wb.Sheets(1)
    Dim range As Excel.Range
    Dim s As String

    Dim dgv = DataGridView1
    For i = 0 To dgv.Rows.Count - 2
      For j = 0 To dgv.Columns.Count - 1
        range = ws.Cells(i + 1, j + 1)
        If j <> 2 Then
          s = dgv(j, i).Value.ToString()
        Else
          s = CType(dgv(j, i).Value, Date).ToShortDateString()
        End If
        range.Value2 = s
      Next
    Next
    ws.UsedRange.EntireColumn.AutoFit()
    xlApp.WindowState = Excel.XlWindowState.xlNormal
    xlApp.Visible = True
  End Sub

  Private Sub ButtonExportToTsv_Click(sender As Object,
    e As EventArgs) Handles ButtonExportToTsv.Click

    Dim sb = New System.Text.StringBuilder()
    Dim s As String
    Dim dgv = DataGridView1

    For i = 0 To dgv.Rows.Count - 2
      For j = 0 To dgv.Columns.Count - 1
        Dim v = dgv(j, i).Value
        If j <> 2 Then
          s = v.ToString() + vbTab
        Else
          s = CType(dgv(j, i).Value, Date).ToShortDateString()
        End If
        sb.Append(s)
      Next
      sb.AppendLine()
    Next

    Dim todayString = DateTime.Now.Year.ToString() &
      DateTime.Now.Month.ToString() &
      DateTime.Now.Day.ToString()
    Dim fileName = System.IO.Path.Combine(
      System.Environment.GetFolderPath(
      Environment.SpecialFolder.MyDocuments),
      todayString & "ToExcel.tsv")

    System.IO.File.WriteAllText(fileName,
      sb.ToString(), System.Text.Encoding.
      GetEncoding("Shift_JIS"))
    System.Diagnostics.Process.Start(fileName)
  End Sub
End Class
